@@ -1,29 +1,31 @@
-import abc
-from typing import Iterator, Sequence, Type, TypeVar
+from functools import singledispatchmethod
+from typing import Iterator, Sequence, Type, TypeVar, cast
 
 from event_sourcery.after_commit_subscriber import AfterCommit
-from event_sourcery.dto.raw_event_dict import RawEventDict
-from event_sourcery.event_registry import BaseEventCls, EventRegistry
+from event_sourcery.dto import RawEvent
+from event_sourcery.event_registry import EventRegistry
 from event_sourcery.exceptions import Misconfiguration, NoEventsToAppend
-from event_sourcery.interfaces.event import Event
+from event_sourcery.interfaces.base_event import Event
+from event_sourcery.interfaces.event import Metadata, TEvent
 from event_sourcery.interfaces.outbox_storage_strategy import OutboxStorageStrategy
 from event_sourcery.interfaces.serde import Serde
 from event_sourcery.interfaces.storage_strategy import StorageStrategy
 from event_sourcery.interfaces.subscriber import Subscriber
 from event_sourcery.types.stream_id import StreamId
+from event_sourcery.versioning import NO_VERSIONING, ExplicitVersioning, Versioning
 
 TAggregate = TypeVar("TAggregate")
 
 
-class EventStore(abc.ABC):
+class EventStore:
     def __init__(
         self,
         serde: Serde,
         storage_strategy: StorageStrategy,
         outbox_storage_strategy: OutboxStorageStrategy,
-        event_base_class: Type[BaseEventCls] | None = None,
+        event_base_class: Type[Event] | None = None,
         event_registry: EventRegistry | None = None,
-        subscriptions: dict[Type[Event], list[Subscriber]] | None = None,
+        subscriptions: dict[Type[TEvent], list[Subscriber]] | None = None,
     ) -> None:
         if event_base_class is not None and event_registry is not None:
             raise Misconfiguration(
@@ -47,28 +49,67 @@ class EventStore(abc.ABC):
 
     def load_stream(
         self, stream_id: StreamId, start: int | None = None, stop: int | None = None
-    ) -> list[Event]:
+    ) -> Sequence[Metadata]:
         events = self._storage_strategy.fetch_events(stream_id, start=start, stop=stop)
         return self._deserialize_events(events)
 
+    @singledispatchmethod
     def append(
-        self, stream_id: StreamId, events: Sequence[Event], expected_version: int = 0
+        self,
+        *events: Metadata,
+        stream_id: StreamId,
+        expected_version: int | Versioning = 0,
     ) -> None:
-        if not events:
-            raise NoEventsToAppend
-
-        self._storage_strategy.ensure_stream(
-            stream_id=stream_id, expected_version=expected_version
+        self._append(
+            stream_id=stream_id, events=events, expected_version=expected_version
         )
 
-        serialized_events = self._serialize_events(events, stream_id, expected_version)
-        self._storage_strategy.insert_events(serialized_events)
+    @append.register
+    def _append_events(
+        self,
+        *events: Event,
+        stream_id: StreamId,
+        expected_version: int | Versioning = 0,
+    ) -> None:
+        wrapped_events = self._wrap_events(expected_version, events)
+        self.append(
+            *wrapped_events, stream_id=stream_id, expected_version=expected_version
+        )
 
-        # TODO: make it more robust per subscriber?
+    @singledispatchmethod
+    def _wrap_events(
+        self,
+        expected_version: int,
+        events: Sequence[Event],
+    ) -> Sequence[Metadata]:
+        return [
+            Metadata.wrap(event=event, version=version)
+            for version, event in enumerate(events, start=expected_version + 1)
+        ]
+
+    @_wrap_events.register
+    def _wrap_events_versioning(
+        self, expected_version: Versioning, events: Sequence[Event]
+    ) -> Sequence[Metadata]:
+        return [Metadata.wrap(event=event, version=None) for event in events]
+
+    @singledispatchmethod
+    def publish(
+        self,
+        *events: Metadata,
+        stream_id: StreamId,
+        expected_version: int | Versioning = 0,
+    ) -> None:
+        serialized_events = self._append(
+            stream_id=stream_id, events=events, expected_version=expected_version
+        )
+
+        self._notify(events)
+        self._outbox_storage_strategy.put_into_outbox(serialized_events)
+
+    def _notify(self, events: Sequence[Metadata]) -> None:
         for event in events:
-            # Shall we support async subscribers here?
-            # Shall we support after-commit-subscribers?
-            for subscriber in self._subscriptions.get(type(event), []):
+            for subscriber in self._subscriptions.get(type(event.event), []):
                 if isinstance(subscriber, AfterCommit):
                     self._storage_strategy.run_after_commit(lambda: subscriber(event))
                 else:
@@ -78,20 +119,46 @@ class EventStore(abc.ABC):
             for catch_all_subscriber in catch_all_subscribers:
                 catch_all_subscriber(event)
 
-    def publish(
-        self, stream_id: StreamId, events: Sequence[Event], expected_version: int = 0
+    @publish.register
+    def _publish_events(
+        self,
+        *events: Event,
+        stream_id: StreamId,
+        expected_version: int | Versioning = 0,
     ) -> None:
-        self.append(
-            stream_id=stream_id, events=events, expected_version=expected_version
+        wrapped_events = self._wrap_events(expected_version, events)
+        self.publish(
+            *wrapped_events, stream_id=stream_id, expected_version=expected_version
         )
-        # this happens twice, should be optimized away
-        serialized_events = self._serialize_events(events, stream_id, expected_version)
 
-        # Rethink. Should it be always putting into outbox
-        # once we have async subscribers?
-        self._outbox_storage_strategy.put_into_outbox(serialized_events)
+    def _append(
+        self,
+        stream_id: StreamId,
+        events: Sequence[Metadata],
+        expected_version: int | Versioning,
+    ) -> list[RawEvent]:
+        if not events:
+            raise NoEventsToAppend
 
-    def iter(self, *streams_ids: StreamId) -> Iterator[Event]:
+        new_version = events[-1].version
+        versioning: Versioning
+        if expected_version is not NO_VERSIONING:
+            versioning = ExplicitVersioning(
+                expected_version=cast(int, expected_version),
+                initial_version=cast(int, new_version),
+            )
+        else:
+            versioning = NO_VERSIONING
+
+        self._storage_strategy.ensure_stream(
+            stream_id=stream_id,
+            versioning=versioning,
+        )
+        serialized_events = self._serialize_events(events, stream_id)
+        self._storage_strategy.insert_events(serialized_events)
+        return serialized_events
+
+    def iter(self, *streams_ids: StreamId) -> Iterator[Metadata]:
         events_iterator = self._storage_strategy.iter(*streams_ids)
         for event in events_iterator:
             yield self._serde.deserialize(
@@ -101,16 +168,15 @@ class EventStore(abc.ABC):
     def delete_stream(self, stream_id: StreamId) -> None:
         self._storage_strategy.delete_stream(stream_id)
 
-    def save_snapshot(self, stream_id: StreamId, snapshot: Event, version: int) -> None:
+    def save_snapshot(self, stream_id: StreamId, snapshot: Metadata) -> None:
         serialized = self._serde.serialize(
             event=snapshot,
             stream_id=stream_id,
-            name=self._event_registry.name_for_type(type(snapshot)),
-            version=version,
+            name=self._event_registry.name_for_type(type(snapshot.event)),
         )
         self._storage_strategy.save_snapshot(serialized)
 
-    def _deserialize_events(self, events: list[RawEventDict]) -> list[Event]:
+    def _deserialize_events(self, events: list[RawEvent]) -> list[Metadata]:
         return [
             self._serde.deserialize(
                 event=event,
@@ -120,14 +186,15 @@ class EventStore(abc.ABC):
         ]
 
     def _serialize_events(
-        self, events: Sequence[Event], stream_id: StreamId, expected_stream_version: int
-    ) -> list[RawEventDict]:
+        self,
+        events: Sequence[Metadata],
+        stream_id: StreamId,
+    ) -> list[RawEvent]:
         return [
             self._serde.serialize(
                 event=event,
                 stream_id=stream_id,
-                name=self._event_registry.name_for_type(type(event)),
-                version=version,
+                name=self._event_registry.name_for_type(type(event.event)),
             )
-            for version, event in enumerate(events, start=expected_stream_version + 1)
+            for event in events
         ]
