@@ -1,31 +1,36 @@
 import abc
 from typing import Callable, Iterator
 
-from event_sourcery.dto.raw_event_dict import RawEventDict
+from event_sourcery.dto import RawEvent
 from event_sourcery.types.stream_id import StreamId
+from event_sourcery.versioning import Versioning
 
 
 class StorageStrategy(abc.ABC):
     @abc.abstractmethod
     def fetch_events(
         self, stream_id: StreamId, start: int | None = None, stop: int | None = None
-    ) -> list[RawEventDict]:
+    ) -> list[RawEvent]:
         pass
 
     @abc.abstractmethod
-    def iter(self, *stream_ids: StreamId) -> Iterator[RawEventDict]:
+    def iter(self, *stream_ids: StreamId) -> Iterator[RawEvent]:
         pass
 
     @abc.abstractmethod
-    def insert_events(self, events: list[RawEventDict]) -> None:
+    def insert_events(self, events: list[RawEvent]) -> None:
         pass
 
     @abc.abstractmethod
-    def save_snapshot(self, snapshot: RawEventDict) -> None:
+    def save_snapshot(self, snapshot: RawEvent) -> None:
         pass
 
     @abc.abstractmethod
-    def ensure_stream(self, stream_id: StreamId, expected_version: int) -> None:
+    def ensure_stream(
+        self,
+        stream_id: StreamId,
+        versioning: Versioning,
+    ) -> None:
         pass
 
     @abc.abstractmethod
