@@ -16,7 +16,7 @@ class Stream:
     __tablename__ = "event_sourcery_streams"
 
     uuid = Column(GUID(), primary_key=True)
-    version = Column(BigInteger(), nullable=False)
+    version = Column(BigInteger(), nullable=True)
 
 
 class Event:
@@ -31,12 +31,12 @@ class Event:
     )
 
     id = Column(BigInteger().with_variant(Integer(), "sqlite"), primary_key=True)
-    version = Column(Integer(), nullable=False)
+    version = Column(Integer(), nullable=True)
     uuid = Column(GUID(), index=True, unique=True)
     stream_id = Column(GUID(), nullable=False, index=True)
-    name = Column(String(50), nullable=False)
+    name = Column(String(200), nullable=False)
     data = Column(JSONB(), nullable=False)
-    event_metadata = Column(JSONB(), nullable=False)
+    event_context = Column(JSONB(), nullable=False)
     created_at = Column(DateTime(), nullable=False, index=True)
 
 
@@ -48,7 +48,7 @@ class Snapshot:
     stream_id = Column(GUID(), nullable=False, index=True)
     name = Column(String(50), nullable=False)
     data = Column(JSONB(), nullable=False)
-    event_metadata = Column(JSONB(), nullable=False)
+    event_context = Column(JSONB(), nullable=False)
     created_at = Column(DateTime(), nullable=False)
 
 
